import contextlib
import tkinter
from tkinter import ttk, N, S, E, W
from chronophore import config, controller, utils


class ChronophoreUI():
    """Simple Tkinter GUI for chronophore :
            - Entry for user id input
            - Button to sign in or out
            - List of currently signed in users
    """

    def __init__(self, timesheet):
        self.t = timesheet

        self.root = tkinter.Tk()
        self.root.title("STEM Sign In")
        self.content = ttk.Frame(self.root, padding=(5, 5, 10, 10))

        # variables
        self.signed_in = tkinter.StringVar()
        self.user_id = tkinter.StringVar()
        self.feedback = tkinter.StringVar()

        # widgets
        self.frm_signedin = ttk.Frame(
            self.content,
            borderwidth=5,
            relief="sunken",
            width=200,
            height=100
        )
        self.lbl_signedin = ttk.Label(
            self.content,
            text="Currently Signed In:"
        )
        self.lbl_signedin_list = ttk.Label(
            self.frm_signedin,
            textvariable=self.signed_in
        )
        self.lbl_welcome = ttk.Label(
            self.content,
            text=config.GUI_WELCOME_LABLE
        )
        self.lbl_id = ttk.Label(self.content, text="Enter Student ID")
        self.ent_id = ttk.Entry(
            self.content,
            textvariable=self.user_id)
        self.lbl_feedback = ttk.Label(self.content, textvar=self.feedback)
        self.btn_sign = ttk.Button(
            self.content,
            text="Sign In/Out",
            command=self._sign_in_button_press
        )

        # assemble grid
        self.content.grid(column=0, row=0, sticky=(N, S, E, W))
        self.lbl_signedin.grid(column=0, row=0, pady=0, sticky=(W))
        self.frm_signedin.grid(
            column=0, row=1, columnspan=1, rowspan=3, sticky=(N, S, E, W)
        )
        self.lbl_signedin_list.grid(column=0, row=0, columnspan=1, rowspan=3)
        self.lbl_welcome.grid(column=2, row=1, columnspan=1)
        # TODO(amin): figure out why lbl_id and btn_sign wiggle
        # when lbl_signedin_list updates
        self.lbl_id.grid(column=2, row=2, columnspan=1, sticky=(N))
        # TODO(amin): add select all shortcuts to this entry
        self.ent_id.grid(column=2, row=2, columnspan=1, sticky=(E, W))
        self.lbl_feedback.grid(column=2, row=2, sticky=(S))
        self.btn_sign.grid(column=2, row=3, columnspan=1, sticky=(N))

        # resize weights
        self.root.columnconfigure(0, minsize=400, weight=1)
        self.root.rowconfigure(0, minsize=200, weight=1)
        self.content.columnconfigure(0, weight=1)
        self.content.columnconfigure(1, weight=3)
        self.content.columnconfigure(2, weight=3)
        self.content.columnconfigure(3, weight=3)
        self.content.rowconfigure(0, weight=0)
        self.content.rowconfigure(1, weight=3)
        self.content.rowconfigure(2, minsize=100, weight=1)
        self.content.rowconfigure(3, weight=3)

        self.root.bind('<Return>', self._sign_in_button_press)
        self.root.bind('<KP_Enter>', self._sign_in_button_press)
        self.ent_id.focus()

        self._set_signed_in()

        self.root.mainloop()

    def _set_signed_in(self):
        names = [
            " ".join([first, last])
            for first, last in controller.signed_in_names(self.t)
        ]
        self.signed_in.set('\n'.join(sorted(names)))

    def _show_feedback(self, message, seconds=None):
        """Display a message in lbl_feedback, which then times out
        after some number of seconds. Use after() to schedule a callback
        to hide the feedback message. This works better than using threads,
        which can cause problems in Tk.
        """
        if seconds is None:
            seconds = config.MESSAGE_DURATION

        # cancel any existing callback to clear the feedback
        # label. this prevents flickering and inconsistent
        # timing during rapid input.
        with contextlib.suppress(AttributeError):
            self.root.after_cancel(self.clear_feedback)

        self.feedback.set(message)
        self.clear_feedback = self.root.after(
            1000 * seconds, lambda: self.feedback.set("")
        )

    def _sign_in_button_press(self, *args):
        """Validate input from ent_id, then sign in to the Timesheet."""
        user_id = self.ent_id.get().strip()

        try:
            sign_in_status = controller.sign(user_id, self.t)
        except (ValueError, FileNotFoundError) as e:
            self._show_feedback(e)
        else:
            user_name = " ".join(utils.user_name(user_id, utils.get_users()))
            self._show_feedback("{}: {}".format(sign_in_status, user_name))
        finally:
            self._set_signed_in()

        self.ent_id.delete(0, 'end')
        self.ent_id.focus()


if __name__ == '__main__':
    # Usage example
    from chronophore.model import Timesheet
    t = Timesheet()
    ui = ChronophoreUI(timesheet=t)



import logging

from chronophore.view import ChronophoreUI
from chronophore.model import Timesheet


def main():
    logger = logging.getLogger(__name__)

    logger.debug("Program initialized")

    ChronophoreUI(timesheet=Timesheet())

    logger.debug("Program stopping")

if __name__ == '__main__':
    main()



#!/usr/bin/python3

"""This script extracts data from an xlsx file and
saves it as a Chronophore-compatible json file.
"""

import argparse
import json
import logging
import openpyxl
import pathlib
from collections import OrderedDict
from datetime import datetime


def get_args():
    parser = argparse.ArgumentParser(
        description="Convert Chronophore data from xlsx to json."
    )

    parser.add_argument('input', help="path of excel file to use as input")

    parser.add_argument(
        '-o', '--output',
        help="path of json file to generate (default: ./$(input_file_name).json)"
    )
    parser.add_argument(
        '-s', '--sheet',
        help="name of sheet in excel spreadsheet (default: use first sheet)"
    )
    parser.add_argument(
        '-c', '--clobber', action="store_true",
        help="overwrite output file if it exists"
    )
    parser.add_argument(
        '-S', '--sort-keys', action="store_true",
        help="sort keys in json output"
    )

    return parser.parse_args()


def excel_to_data(worksheet):
    """Extract and return data from an excel worksheet.
    Preserve ordere by useing an OrderedDict. The values
    in the first column are used as keys, and the values
    in the first row are used as headers.
    """

    data = OrderedDict()

    # NOTE(amin): worksheet.rows is a generator because
    # the workbook was opened with read_only=True. Thus,
    # we use next() (instead of a subscript) to get the
    # first row.
    key_header, *headers = tuple(cell.value for cell in next(worksheet.rows))
    logging.debug("Headers: {}".format(headers))

    for row_num, row in enumerate(worksheet.rows):
        if row_num == 0:
            continue
        entry = OrderedDict()
        key_cell, *value_cells = row
        for header, cell in zip(headers, value_cells):
            if (cell.value is not None
                and cell.parent is not None
                and cell.is_date):  # openpyxl issue #625
                value = datetime.strftime(cell.value, "%Y-%m-%d")
            else:
                value = cell.value
            entry[header] = value
        data[key_cell.value] = entry

    return data


if __name__ == '__main__':
    args = get_args()

    EXCEL_FILE = pathlib.Path(args.input)
    name = EXCEL_FILE.name

    if args.output:
        JSON_FILE = pathlib.Path(args.output)
    else:
        JSON_FILE = pathlib.Path('.', EXCEL_FILE.stem + '.json')

    CLOBBER = args.clobber
    SORT_KEYS = args.sort_keys

    if not CLOBBER and JSON_FILE.exists():
        logging.warning(
            "{} exists. To overwrite it, use '--clobber'.".format(JSON_FILE)
        )
        raise SystemExit

    wb = openpyxl.load_workbook(
        filename=str(EXCEL_FILE),
        read_only=True,
        data_only=True
    )
    WS = wb[args.sheet] if args.sheet else wb.worksheets[0]

    data = excel_to_data(WS)

    with JSON_FILE.open('w') as f:
        json.dump(data, f, indent=4, sort_keys=SORT_KEYS)
    logging.info("Json saved: {}".format(JSON_FILE))



"""Chronophore is a simple time-tracking program. It keeps track of
users' hours as they sign in and out. Data is stored in a
human-readable json file.

This project was started to help keep track of students signing in
and out at a tutoring program in a community college, but should be
adaptable to other use cases.
"""

import logging

# Information
__license__ = 'MIT'
__version__ = '0.2.1'
__author__ = 'Amin Mesbah'
__email__ = 'mesbahamin@gmail.com'

# Set up logger
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)

fh = logging.FileHandler('debug.log')
fh.setLevel(logging.DEBUG)

ch = logging.StreamHandler()
ch.setLevel(logging.WARNING)

formatter = logging.Formatter(
    "{asctime} {levelname} ({name}): {message}", style='{'
)
fh.setFormatter(formatter)
ch.setFormatter(formatter)

logger.addHandler(fh)
logger.addHandler(ch)
logging.basicConfig(level=logging.DEBUG)



import logging
from datetime import datetime

from chronophore import model, utils

logger = logging.getLogger(__name__)


def signed_in_names(timesheet):
    """Return list of names of currently signed in users."""
    return [
        tuple(timesheet[k].name.split())
        for k in timesheet.signed_in
    ]


def user_signed_in(user_id, timesheet):
    """Check whether a given user is signed in.

    Return:
        - None if 0 instances of user_id signed in
        - Key if 1 instance of user_id signed in

    Raise:
        - ValueError if >1 instance of user_id signed in
    """
    [key] = (
        [
            k for k in timesheet.signed_in
            if timesheet.sheet[k]['user_id'] == user_id
        ]
        or [None]
    )
    return key


def sign_in(user_id, name, date=None, time_in=None):
    now = datetime.today()
    if date is None:
        date = datetime.strftime(now, "%Y-%m-%d")
    if time_in is None:
        time_in = datetime.strftime(now, "%H:%M:%S")

    signed_in_entry = model.Entry(
        date=date,
        name=name,
        time_in=time_in,
        time_out=None,
        user_id=user_id,
    )

    logger.debug("Entry object initialized: {}".format(repr(signed_in_entry)))
    return signed_in_entry


def sign_out(entry, time_out=None):
    if time_out is None:
        time_out = datetime.strftime(datetime.today(), "%H:%M:%S")

    signed_out_entry = entry._replace(time_out=time_out)

    logger.info("Entry signed out: {}".format(repr(signed_out_entry)))
    return signed_out_entry


def sign(user_id, timesheet, users_file=None):
    """Check user id for validity, then sign user in or out
    depending on whether or not they are currently signed in.

    Return:
        - status: "Signed in", "Signed out"

    Raise:
        - ValueError if user_id is invalid. Include a
        message to be passed to the caller.
    """

    users = utils.get_users(users_file)

    if not utils.is_valid(user_id):
        logger.debug("Invalid input: {}".format(user_id))
        raise ValueError(
            "Invalid Input: {}".format(user_id)
        )

    elif not utils.is_registered(user_id, users):
        logger.debug("User not registered: {}".format(user_id))
        raise ValueError(
            "{} not registered. Please register at the front desk.".format(
                user_id
            )
        )

    try:
        key = user_signed_in(user_id, timesheet)
    except ValueError:
        # handle duplicates
        duplicate_entry_keys = [
            k for k in timesheet.signed_in
            if timesheet.sheet[k]['user_id'] == user_id
        ]
        logger.warning(
            "Multiple signed in instances of user {}: {}".format(
                user_id, duplicate_entry_keys
            )
        )
        logger.info(
            "Signing out of multiple instances of user {}: {}".format(
                user_id, duplicate_entry_keys
            )
        )
        for key in duplicate_entry_keys:
            timesheet[key] = sign_out(timesheet[key])

        raise ValueError(
            "Signing out of multiple instances of user {}".format(
                user_id
            )
        )
    else:
        if not key:
            user_name = ' '.join(
                utils.user_name(user_id, utils.get_users(users_file))
            )
            key = utils.new_key()
            timesheet[key] = sign_in(user_id, user_name)
            status = "Signed in"
        else:
            timesheet[key] = sign_out(timesheet[key])
            status = "Signed out"

        return status



from pathlib import Path

# Files
DATA_DIR = Path.home() / '.local' / 'share' / 'chronophore'
USERS_FILE = DATA_DIR.joinpath('users.json')

# GUI Settings
USER_ID_LENGTH = 9
MESSAGE_DURATION = 5
GUI_WELCOME_LABLE = "Welcome to the STEM Learning Center!"



#!/usr/bin/python3

import argparse
import json
import logging
import openpyxl
import pathlib
from collections import OrderedDict


def get_args():
    parser = argparse.ArgumentParser(
        description="Convert Chronophore data from json to xlsx."
    )

    parser.add_argument('input', help="path of json file to use as input")

    parser.add_argument(
        '-o', '--output',
        help="path of excel file to generate (default: ./$(input_file_name).xlsx)"
    )
    parser.add_argument(
        '-c', '--clobber', action="store_true",
        help="overwrite output file if it exists"
    )

    return parser.parse_args()


def data_to_excel(data, output_file):
    """Saves data from one of Chronophore's json files
    to an Excel spreadsheet.

    Parameters:
        - data: A dictionary (preferably an OrderedDict) from
                a json file created by Chronophore.
        - title: The title of the output file.
    """
    try:
        headers = {header for key in data for header in data[key].keys()}
        logging.debug("Headers: {}".format(headers))
    except (TypeError, AttributeError) as e:
        logging.error(
            "Invalid data. Must be a valid dictionary: {} ({})".format(data, e)
        )
    else:
        # assign columns to headers
        header_columns = {
            header: column for key in data
            for column, header in enumerate(headers, start=2)
        }

        # create sheet
        wb = openpyxl.Workbook()
        sheet = wb.active
        sheet.title = output_file.name

        # make headers
        sheet.cell(row=1, column=1).value = "Key"
        for (header, col_num) in header_columns.items():
            sheet.cell(row=1, column=col_num).value = header

        sheet.freeze_panes = 'A2'

        # fill in data
        for row_num, key in enumerate(data.keys(), start=2):
            sheet.cell(row=row_num, column=1).value = key
            for header, value in data[key].items():
                col_num = header_columns[header]
                sheet.cell(row=row_num, column=col_num).value = value

        return wb


if __name__ == '__main__':
    args = get_args()

    CLOBBER = args.clobber

    JSON_FILE = pathlib.Path(args.input)

    if args.output:
        EXCEL_FILE = pathlib.Path(args.output)
    else:
        EXCEL_FILE = pathlib.Path('.', JSON_FILE.stem + '.xlsx')

    if not CLOBBER and EXCEL_FILE.exists():
        logging.warning(
            "'{}' exists. To overwrite it, use '--clobber'.".format(EXCEL_FILE)
        )
        raise SystemExit

    with JSON_FILE.open('r') as f:
        data = json.load(f, object_pairs_hook=OrderedDict)

    wb = data_to_excel(data, EXCEL_FILE)
    wb.save(str(EXCEL_FILE))
    logging.info("Worksheet saved: {}".format(EXCEL_FILE))



import collections
import json
import logging
import pathlib
import shutil
import uuid

import chronophore
from chronophore import config

logger = logging.getLogger(__name__)


def validate_json(json_file):
    """Raise exception if json_file contains errors or key collisions."""
    try:
        with json_file.open('r') as f:
            d = json.load(f, object_pairs_hook=list)
    except json.decoder.JSONDecodeError as e:
        logger.critical("Invalid json file: {}.".format(e))
        raise
    else:
        keys = [key for key, value in d]

        key_collisions = [
            key for key, count
            in collections.Counter(keys).items()
            if count > 1
        ]

        if key_collisions:
            message = "Key collision(s): {} in json file: {}".format(
                key_collisions, json_file
            )
            logger.warning(message)
            raise ValueError(message)
        else:
            logger.debug("{} validated".format(json_file))


def get_users(users_file=None):
    if users_file is None:
        users_file = config.USERS_FILE

    try:
        validate_json(users_file)
    except FileNotFoundError as e:
        logger.warning('{}. Copying example users file.'.format(e))
        default_config_dir = pathlib.Path(
            pathlib.Path(chronophore.__file__).parent,
            'config'
        )
        default_users = default_config_dir.joinpath('users.json')
        config.DATA_DIR.mkdir(exist_ok=True, parents=True)
        shutil.copy(str(default_users), str(config.DATA_DIR))
        logger.info('New users file: {}'.format(users_file))

    with users_file.open('r') as f:
        users = json.load(f)
    return users


def user_name(user_id, users):
    return (users[user_id]['First Name'], users[user_id]['Last Name'])


def is_valid(user_id):
    try:
        int(user_id)
    except ValueError:
        return False
    else:
        return bool(len(user_id) == config.USER_ID_LENGTH)


def is_registered(user_id, users):
    registered_ids = list(users.keys())
    return bool(user_id in registered_ids)


def new_key():
    """Generate a UUID version 4 (basically random)"""
    return str(uuid.uuid4())



import io
import os
import re
from setuptools import setup


here = os.path.abspath(os.path.dirname(__file__))


def read(*names, **kwargs):
    with io.open(
        os.path.join(here, *names),
        encoding=kwargs.get('encoding', 'utf8')
    ) as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(
        r"^__version__ = ['\"]([^'\"]*)['\"]",
        version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


with io.open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
        long_description = f.read()


setup(
    name='chronophore',
    version=find_version("chronophore", "__init__.py"),

    description='Simple desktop app for tracking sign-ins and sign-outs in a tutoring center.',

    long_description=long_description,

    url='https://github.com/mesbahamin/chronophore',

    # Author details
    author='Amin Mesbah',
    author_email='mesbahamin@gmail.com',

    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        # 3 - Alpha
        # 4 - Beta
        # 5 - Production/Stable
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
        'Topic :: Office/Business :: Scheduling',

        'License :: OSI Approved :: MIT License',

        'Natural Language :: English',

        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
    ],

    keywords='database desktop education json kiosk sign-in time',

    packages=['chronophore'],
    entry_points={
        'gui_scripts': [
            'chronophore=chronophore.__main__:main',
        ],
    },

    scripts=['scripts/excel_to_json.py', 'scripts/json_to_excel.py'],
    package_data={'chronophore': ['config/*']},
    install_requires=['openpyxl'],

    extras_require={
        'dev': ['flake8'],
        'test': ['pytest'],
    },
)



import collections
import json
import logging
import os
from datetime import datetime

import chronophore

logger = logging.getLogger(__name__)


Entry = collections.namedtuple('Entry', 'date name time_in time_out user_id')


class Timesheet():
    """Contains multiple entries"""

    def __init__(self, data_file=None):
        self.sheet = collections.OrderedDict()
        self.signed_in = []
        data_dir = chronophore.config.DATA_DIR

        if data_file is None:
            today = datetime.strftime(datetime.today(), "%Y-%m-%d")
            file_name = today + ".json"
            data_file = data_dir.joinpath(file_name)

        self.data_file = data_file

        try:
            chronophore.utils.validate_json(self.data_file)
        except FileNotFoundError:
            logger.debug(
                "{} not found. It will be created.".format(data_file)
            )
        except json.decoder.JSONDecodeError as e:
            backup = data_file.with_suffix('.bak')
            os.rename(str(data_file), str(backup))
            logger.error("{}. {} moved to {}".format(e, data_file, backup))
        else:
            with data_file.open('r') as f:
                self.load_sheet(data=f)

        logger.debug("Timesheet object initialized.")
        logger.debug("Timesheet data file: {}".format(self.data_file))

        self._update_signed_in()

    def __iter__(self):
        return iter(self.sheet)

    def __contains__(self, key):
        return key in self.sheet.keys()

    def __len__(self):
        return len(self.sheet)

    def __getitem__(self, key):
        """Load and return an entry object."""
        entry = Entry(
            date=self.sheet[key]['date'],
            name=self.sheet[key]['name'],
            time_in=self.sheet[key]['time_in'],
            time_out=self.sheet[key]['time_out'],
            user_id=self.sheet[key]['user_id'],
        )
        logger.debug("Entry loaded: {}".format(repr(entry)))
        return entry

    def __setitem__(self, key, entry):
        """Format an entry as an ordered dict and
        add it to the timesheet.
        """
        self.sheet[key] = entry._asdict()
        logger.debug("Entry saved: {}".format(repr(entry)))
        self._update_signed_in()
        self.save_sheet()

    def _update_signed_in(self):
        """Update the list of all entries that haven't been signed out."""
        self.signed_in = [
            k for k, v in self.sheet.items() if v['time_out'] is None
        ]
        logger.debug("Signed in entries: {}".format(self.signed_in))

    def load_sheet(self, data):
        """Read the timesheet from a json file."""
        self.sheet = json.load(data, object_pairs_hook=collections.OrderedDict)
        logger.debug("Sheet loaded")
        self._update_signed_in()

    def save_sheet(self, data_file=None):
        """Write the timesheet to json file."""
        if data_file is None:
            data_file = self.data_file

        data_file.parent.mkdir(exist_ok=True, parents=True)
        with data_file.open('w') as f:
            json.dump(self.sheet, f, indent=4, sort_keys=False)
        logger.debug("Sheet saved to {}".format(data_file.resolve()))
